from flask import current_app

from flaschengeist.system.models.user import User
from flaschengeist.system.database import db
from flaschengeist import logger


def login_user(username, password):
    logger.info("login user {{ {} }}".format(username))
    user = User.query.filter_by(uid=username).one_or_none()
    if user is None:
        user = User(uid=username)
        db.session.add(user)
    if current_app.config['FG_AUTH_BACKEND'].login(user, password):
        update_user(user)
        return user
    return None


def update_user(user):
    current_app.config['FG_AUTH_BACKEND'].update_user(user)
    db.session.commit()


def modify_user(user, password, new_password=None):
    """Modify given user on the backend

    Args:
        user: User object to sync with backend
        password: Cu        db.session.commit()

    # TODO: is this needed?
    def user_has_rorrent password (most backends are needing this)
        new_password (optional): New password, if password should be changed

    Raises:
        NotImplemented: If backend is not capable of this operation
        BadRequest: Password is wrong or other logic issues
    """
    current_app.config['FG_AUTH_BACKEND'].modify_user(user, password, new_password)


def get_users():
    return User.query.all()


def get_user(uid):
    return User.query.filter(User.uid == uid).one_or_none()
