from ..database import db


class Event(db.Model):
    """Model for an Event"""
    __tablename__ = 'event'
    id = db.Column(db.Integer, primary_key=True)
    begin = db.Column(db.DateTime, nullable=False)
    end = db.Column(db.DateTime)
    description = db.Column(db.String(240))
    kind_id = db.Column(db.Integer, db.ForeignKey('event_kind.id', ondelete="CASCADE"), nullable=False)
    kind = db.relationship("EventKind")
    slots = db.relationship("EventSlot", back_populates="event", cascade="all, delete")
    #notices = db.relationship("EventNotice", back_populates="event")

    def serialize(self):
        return {
            "id": self.id,
            "begin": self.begin,
            "end": self.end,
            "description": self.description,
            "kind": self.kind
        }


class EventKind(db.Model):
    """Model for an EventKind"""
    __tablename__ = "event_kind"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(30), nullable=False, unique=True)

    def serialize(self):
        return {
            "id": self.id,
            "name": self.name
        }


class EventSlot(db.Model):
    """Model for an EventSlot"""
    __tablename__ = "event_slot"
    id = db.Column(db.Integer, primary_key=True)
    start = db.Column(db.DateTime)
    end = db.Column(db.DateTime)
    event_id = db.Column(db.Integer, db.ForeignKey('event.id'), nullable=False)
    event = db.relationship("Event", back_populates="slots")
    slots = db.relationship("JobSlot", back_populates="event_slot")

    def serialize(self):
        return {
            "id": self.id,
            "start": self.start,
            "end": self.end,
            "event": self.event_id,
        }


class JobSlot(db.Model):
    __tablename__ = "job_slot"
    id = db.Column(db.Integer, primary_key=True)
    needed_persons = db.Column(db.Numeric(precision=4, scale=2))
    event_slot_id = db.Column(db.Integer, db.ForeignKey('event_slot.id'))
    event_slot = db.relationship("EventSlot", back_populates="slots")
    kind_id = db.Column(db.Integer, db.ForeignKey('job_kind.id'))
    kind = db.relationship("JobKind")
    jobs = db.relationship("Job", back_populates="slot")


class Job(db.Model):
    __tablename__ = "job"
    id = db.Column(db.Integer, primary_key=True)
    value = db.Column(db.Numeric(precision=3, scale=2))
    user_id = db.Column(db.Integer, db.ForeignKey('user.id'))
    user = db.relationship("User")
    slot_id = db.Column(db.Integer, db.ForeignKey('job_slot.id'))
    slot = db.relationship("JobSlot")


class JobKind(db.Model):
    __tablename__ = "job_kind"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(30))
